#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <switch.h>

#ifdef DISPLAY_IMAGE
#include "image_bin.h"
#endif

#define FB_WIDTH  1280
#define FB_HEIGHT 720

#define MAX_COL 50 //maximum number of custom colors

/*
Remove above and uncomment below for 1080p
#define FB_WIDTH  1920
#define FB_HEIGHT 1080

>1080p strobing

*/
int main(int argc, char* argv[])
{
	NWindow* win = nwindowGetDefault();
	
	Framebuffer fb;
	framebufferCreate(&fb, win, FB_WIDTH, FB_HEIGHT, PIXEL_FORMAT_RGBA_8888, 2);
	framebufferMakeLinear(&fb);

#ifdef DISPLAY_IMAGE
	u8* imageptr = (u8*)image_bin;
#endif

	FILE *colorFile = fopen("colors.txt", "r"); //try to open the file

	int lines = 8;
	bool colorExist = false; //used for logic depending on the existence of the file

	if (colorFile != NULL) { //count number of lines in file if it exists
		lines = 1;
		colorExist = true;
		int chr = fgetc(colorFile); //fgetc is an int, not char. 
		while ((chr != EOF) && (lines < MAX_COL)) { //go through file characterwise and see if any are newlines until End or MAX_COL, whichever is first
			if (chr == '\n') { //																					  Of
				lines++; //																							  File
			}
			chr = fgetc(colorFile);
		}
		fseek(colorFile, 0, SEEK_SET); //reset file stream pointer thingy's position to the beginning of the file
	}
	
	int colors[lines]; //assume number of lines equals number of colors, make array of that dimension

	if (colorExist) { //if the file exists, use its values. if not, use generic color array
		for (int i = 0; i < lines; i++) { //define each element as a line from the file one at a time
			fscanf(colorFile, "%x ", &colors[i]); // %x for hex, the space after is for newline detection
		}
	} else {
		int a, b, j;
		for (int i = 0; i < lines; i++) { //generate a value for each element one at a time
			j = i + 1;
			a = (((i % 2) + ((i / 2) % 2)) + (((i / 4)*i) % 3) + (i / 6) + (i / 4)) % 2;
			b = (((j % 2) + ((j / 2) % 2)) + (((j / 4)*j) % 3) + (j / 6) + (j / 4) + 1) % 2;
			colors[i] = 0x11*a + 0x1100*b + 0x110000*(i/4); //integer division is just truncate/floor round
		}
	}

	fclose(colorFile); //close the file

	u32 cnt = 0;
	int duration = 10; //number of frames each color is active for
	int colSize = sizeof(colors) / sizeof(colors[0]); //how many elements in color array, could probably get away with just using the 'lines' variable from earlier
	int col1 = 0; //first color's index
	int col2 = colSize - 1; //second color's index
	int bright1 = 15; //how bright first color is, only used with generic color array
	int bright2 = 15; //how bright second color is, only used with generic color array
	bool isOn = true; //bool used for switching between first and second color

	while (appletMainLoop())
	{
		hidScanInput();

		u64 kDown = hidKeysDown(CONTROLLER_P1_AUTO); //switch player one controller input
		
		if (kDown & KEY_PLUS) { //safely exit app
			break;
		}
		
		if (kDown & KEY_MINUS) { //reset timing values
			duration = 10;
			cnt = 0;
		}

		if ((kDown & KEY_DUP) && (60 - duration)) { //increase color duration
			duration++;
		}

		if ((kDown & KEY_DDOWN) && (duration - 1)) { //decrease color duration
			duration--;
		}

		if (kDown & KEY_RIGHT) { //reset color one brightness
			bright1 = 15;
		}

		if (kDown & KEY_LEFT) { //reset color two brightness
			bright2 = 15;
		}
		
		if (kDown & KEY_A) { //cycle color one positively through array
			col1 = (col1 + 1) % colSize;
		}

		if (kDown & KEY_Y) { //cycle color one negatively through array
			col1 = (col1 - 1 + colSize) % colSize;
		}
		
		if (kDown & KEY_X) { //cycle color two positively through array
			col2 = (col2 + 1) % colSize;
		}
		
		if (kDown & KEY_B) { //cycle color two negatively through array
			col2 = (col2 - 1 + colSize) % colSize;
		}
		
		if ((kDown & KEY_R) && (15 - bright1)) { //increase color one brightness
			bright1++;
		}

		if ((kDown & KEY_ZR) && bright1) { //decrease color one brightness
			bright1--;
		}
		
		if ((kDown & KEY_L) && (15 - bright2)) { //increase color two brightness
			bright2++;
		}

		if ((kDown & KEY_ZL) && bright2) { //decrease color two brightness
			bright2--;
		}
		
		u32 stride;
		u32* framebuf = (u32*)framebufferBegin(&fb, &stride);

		cnt = (cnt + 1) % (duration + 1); //frame counter, used to determine when to change colors
		if (cnt == 0) { //change colors
			isOn = !isOn;
			//col1 = (col1 + 1) % colSize;
		}

		for (u32 y = 0; y < FB_HEIGHT; y++)
		{
			for (u32 x = 0; x < FB_WIDTH; x++)
			{
				u32 pos = y * stride / sizeof(u32) + x;
#ifdef DISPLAY_IMAGE
				framebuf[pos] = RGBA8_MAXALPHA(imageptr[pos * 3 + 0] + (cnt * 4), imageptr[pos * 3 + 1], imageptr[pos * 3 + 2]);
#else
				
				if (isOn && colorExist) { framebuf[pos] = colors[col1]; }
				if (isOn && !colorExist) { framebuf[pos] = colors[col1] * bright1; }
				if (!isOn && colorExist) { framebuf[pos] = colors[col2]; }
				if (!isOn && !colorExist) { framebuf[pos] = colors[col2] * bright2; }
				

				//framebuf[pos] = colors[col1] * bright1;
#endif
			}
		}
		
		framebufferEnd(&fb);
	}

	framebufferClose(&fb);
	return 0;
}